import re
import random
import datetime
import sys
import string
import logging

import pytz

from ramjet.settings import LOG_NAME, LOG_PATH
from .jinja import debug_wrapper, TemplateRendering


log = logging.getLogger(LOG_NAME)
__all__ = [
    'utcnow', 'setup_log', 'validate_email', 'validate_mobile', 'generate_random_string',
    'debug_wrapper', 'TemplateRendering',
]
UTC = pytz.timezone('utc')


def utcnow():
    return datetime.datetime.utcnow().replace(tzinfo=UTC)


def setup_log():
    _format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    formatter = logging.Formatter(_format)
    # set stdout
    ch = logging.StreamHandler(sys.stdout)
    ch.setLevel(logging.DEBUG)
    ch.setFormatter(formatter)
    # set log file
    fh = logging.FileHandler(LOG_PATH)
    fh.setLevel(logging.INFO)
    fh.setFormatter(formatter)
    # log
    logging.getLogger(LOG_NAME).setLevel(logging.DEBUG)
    log = logging.getLogger()
    log.setLevel(logging.DEBUG)
    log.addHandler(ch)
    # log.addHandler(sh)
    log.addHandler(fh)


def validate_email(email):
    epat = re.compile(r'^[_a-z0-9-]+(\.[_a-z0-9-]+)*'
                      r'@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,4})$')
    return epat.match(email)


def validate_mobile(mobile):
    ippat = re.compile(r'^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$')
    return ippat.match(mobile)


def generate_random_string(length):
    alphbet = string.ascii_letters + ''.join([str(i) for i in range(10)])
    return ''.join([random.choice(alphbet) for i in range(length)])



from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor
from collections import namedtuple

import tornado.ioloop

from ramjet.settings import N_THREAD_WORKER, N_PROCESS_WORKER


thread_executor = ThreadPoolExecutor(max_workers=N_THREAD_WORKER)
process_executor = ProcessPoolExecutor(max_workers=N_PROCESS_WORKER)
ioloop = tornado.ioloop.IOLoop.instance()



import os
import logging

# server
LISTEN_PORT = 27851

# worker
N_THREAD_WORKER = 8
N_PROCESS_WORKER = 8

# common
CWD = os.path.dirname(__file__)
LOG_NAME = 'ramjet-driver'
LOG_DIR = '/tmp'
LOG_PATH = '{}.log'.format(os.path.join(LOG_DIR, LOG_NAME))
logger = logging.getLogger(LOG_NAME)

# web
OK = 0
ERROR = 1



import os
import glob
import traceback
from collections import namedtuple
from importlib import import_module

from ramjet.settings import logger, CWD


TASK = namedtuple('task', ['name', 'func'])
_TASKS = []


def register_task(task_name, task_func):
    logger.info('register_task for task_name {}'.format(task_name))

    task = TASK(name=task_name, func=task_func)
    _TASKS.append(task)


def detect_tasks():
    logger.info('detect_tasks')

    for taskfpath in glob.glob(os.path.join(CWD, 'tasks', '*')):
        taskfname = os.path.split(taskfpath)[1]
        if taskfname.startswith('__'):
            continue

        module_name = None
        if os.path.isfile(taskfpath):
            if not taskfpath.endswith('.py'):
                continue

            module_name = os.path.splitext(taskfname)[0]
            if module_name == '__init__':
                continue
        else:
            module_name = taskfname

        if not module_name:
            continue

        try:
            logger.debug('try to import ramjet.tasks.{}'.format(module_name))
            module = import_module('ramjet.tasks.{}'.format(module_name))
            task = module.bind_task
        except ImportError:
            logger.error('import module {} error: {}'
                         .format(module_name, traceback.format_exc()))
        except AttributeError:
            logger.error('bind_task not found in {}'.format(module))
        except Exception:
            pass
        else:
            task_name = getattr(task, 'TASK_NAME', module_name)
            register_task(task_name, task)

    return _TASKS



"""Ramjet
"""

import logging
from pathlib import Path

import tornado
from tornado.web import url
from tornado.options import define, options

from ramjet.settings import CWD, LOG_NAME, LISTEN_PORT
from ramjet.utils import setup_log, generate_random_string
from ramjet.views import BaseHandler


log = logging.getLogger(LOG_NAME)
setup_log()
define('port', default=LISTEN_PORT, type=int)
define('debug', default=False, type=bool)


class PageNotFound(BaseHandler):

    @tornado.gen.coroutine
    def get(self, url=None):
        log.debug('GET PageNotFound for url {}'.format(url))

        if url is None:
            self.render2('404.html', url=url)
            self.finish()
            return

        self.redirect_404()


class Application(tornado.web.Application):

    def __init__(self):
        settings = {
            'static_path': str(Path(CWD, 'static')),
            'static_url_prefix': '/static/',
            'template_path': str(Path(CWD, 'templates')),
            'cookie_secret': generate_random_string(50),
            'login_url': '/login/',
            'xsrf_cookies': True,
            'autoescape': None,
            'debug': options.debug
        }
        handlers = [
            # -------------- handler --------------
            url(r'^/404.html$', PageNotFound, name='404'),
        ]
        handlers.append(('/(.*)', PageNotFound))
        self.setup_db()
        self.setup_sentry()
        super(Application, self).__init__(handlers, **settings)



import logging
import traceback

import tornado.httpserver
import tornado.options as opt
from tornado.options import options

from ramjet.app import Application
from ramjet.settings import LOG_NAME
from ramjet.tasks import detect_tasks, _TASKS


logger = logging.getLogger(LOG_NAME)


def main():
    opt.parse_command_line()

    # 暂不需要 Web 应用
    # http_server = tornado.httpserver.HTTPServer(Application())
    # http_server.listen(options.port)

    if options.debug:
        logger.info('start application in debug mode')
        logger.setLevel(logging.DEBUG)
    else:
        logger.info('start application in normal mode')
        logger.setLevel(logging.INFO)
    ioloop = tornado.ioloop.IOLoop.instance()

    detect_tasks()
    for task in _TASKS:
        try:
            logger.info('start {}'.format(task.name))
            task.func()
        except Exception:
            logger.warn('run {} got error: '.format(task.name, traceback.format_exc()))

    ioloop.start()

if __name__ == '__main__':
    main()



import smtplib
from email.mime.text import MIMEText


def send_mail(fr, to, subject, content):
    msg = MIMEText(content)

    msg['Subject'] = subject
    msg['From'] = fr
    msg['To'] = to

    # Send the message via our own SMTP server.
    s = smtplib.SMTP('10.32.135.22', port=25)
    # s.login(user='messerflow@edm.chexiang.com')
    r = s.send_message(msg)
    s.quit()
    return r



import traceback
import datetime
import json

from jinja2 import Environment, FileSystemLoader
from webassets import Environment as AssetsEnvironment
from webassets.ext.jinja2 import AssetsExtension

from ramjet.settings import logger


__all__ = ['debug_wrapper', 'TemplateRendering']


def debug_wrapper(func):
    def wrapper(*args, **kw):
        logger.debug('debug_wrapper for args {}, kw {}'.format(args, kw))
        try:
            yield from func(*args, **kw)
        except Exception:
            self = args[0]
            err_msg = {
                'uri': self.request.uri,
                'version': self.request.version,
                'headers': self.request.headers,
                'cookies': self.request.cookies,
            }
            logger.error('{}\n-----\n{}'.format(
                json.dumps(err_msg, indent=4, sort_keys=True),
                traceback.format_exc(),
            ))
            raise
    return wrapper


class TemplateRendering():

    """
    A simple class to hold methods for rendering templates.
    Copied from
        http://bibhas.in/blog/\
            using-jinja2-as-the-template-engine-for-tornado-web-framework/
    """
    _jinja_env = None
    _assets_env = None

    def render_template(self, template_name, **kw):
        if not self._jinja_env:
            self._jinja_env = Environment(
                loader=FileSystemLoader(self.settings['template_path']),
                extensions=[AssetsExtension]
            )
            self._jinja_env.filters.update({
                'utc2cst': lambda dt: dt + datetime.timedelta(hours=8),
                'jstime2py': lambda ts: datetime.datetime.fromtimestamp(ts / 1000),
                'time_format': lambda dt: datetime.datetime.strftime(dt, '%Y/%m/%d %H:%M:%S'),
            })

        if not self._assets_env:
            self._assets_env = AssetsEnvironment(
                self.settings['static_path'],
                self.settings['static_url_prefix'],
            )
            self._jinja_env.assets_environment = self._assets_env

        template = self._jinja_env.get_template(template_name)
        content = template.render(kw)
        return content



"""Task 示例
"""

from ramjet.settings import logger
from ramjet.engine import ioloop


logger = logger.getChild('tasks.heart')

TASK_NAME = 'heartbeats'


def bind_task():
    def callback(*args, **kw):
        logger.info('tasks heart!')
        ioloop.call_later(10, callback, *args, **kw)

    ioloop.call_later(0, callback)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import json
import logging

import tornado

from ramjet.settings import OK, LOG_NAME
from ramjet.utils import TemplateRendering


log = logging.getLogger(LOG_NAME)
__all__ = ['BaseHandler']


class BaseHandler(tornado.web.RequestHandler, TemplateRendering):

    def get(self, url=None):
        url = url.strip(' /')
        super().get(url)

    @property
    def ip(self):
        return self.request.headers.get('X-Real-IP', self.request.remote_ip)

    def write_json(self, *, status=OK, msg='', data={}):
        j = json.dumps({'status': status, 'msg': msg, 'data': data})
        log.debug('<< {}'.format(j))
        self.write(j)

    def get_argument(self, arg_name, bool=False, *args, **kw):
        val = super().get_argument(arg_name, *args, **kw)
        if bool:
            return False if val in ('false', 'False', False, 0, None) else True
        else:
            return val

    @property
    def is_ajax(self):
        return self.request.headers.get('X-Requested-With') == "XMLHttpRequest"

    @property
    def is_https(self):
        return self.request.headers.get('X-Scheme') == "https"

    def redirect_404(self):
        self.redirect('/404.html')

    def render2(self, template_name, **kwargs):
        """
        This is for making some extra context variables available to
        the template
        """
        content = self.render_template(template_name, **kwargs)
        self.write(content)

    def render_template(self, template_name, **kwargs):
        def static_url(path):
            prefix = self.settings.get('static_url_prefix')
            return os.path.join(prefix, path)

        _kwargs = ({
            'settings': self.settings,
            'static_url': static_url,
            'reverse_url': self.reverse_url,
            'request': self.request,
            'xsrf_token': self.xsrf_token,
            'xsrf_form_html': self.xsrf_form_html,
            'max': max,
            'min': min,
            'is_ajax': self.is_ajax,
            'is_https': self.is_https,
            'current_user': self.current_user,
            'current_app': 'ramjet',
        })
        _kwargs.update(kwargs)
        return super().render_template(template_name, **_kwargs)



from setuptools import setup, find_packages
from pip.req import parse_requirements
from pip.download import PipSession


requires = [str(i.req) for i in parse_requirements('requirements.txt',
                                                   session=PipSession())
            if i.req is not None]


setup(name='ramjet',
      version='1.1.1',
      author="Laisky",
      author_email="ppcelery@gmail.com",
      description="Timing tasks framework.",
      license="MIT/Apache",
      url="https://github.com/Laisky/ramjet",
      keywords="timing tasks frameworks asynchrous",
      packages=find_packages('src'),
      package_dir={'': 'src'},
      include_package_data=True,
      install_requires=requires,
      classifiers=[
          "Development Status :: 4 - Beta",
          'Topic :: Software Development :: Libraries',
          "Environment :: Web Environment",
          "Intended Audience :: Developers",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
          "Programming Language :: Python",
          'Programming Language :: Python :: 3.4',
      ],
      zip_safe=True)
